/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "NamedNodeMap.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNamedNodeMap.h"
#include "gdom/GdomNode.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNamedNodeMap.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateIntGdomNamedNodeMap.h"
#include "gdom/PrivateIntGdomNode.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::NamedNodeMap* obj)
{
    return WebCore::cacheDOMObject<WebCore::NamedNodeMap, GDOMNamedNodeMap>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMNamedNodeMap::GDOMNamedNodeMap(PassRefPtr<WebCore::NamedNodeMap> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMNamedNodeMap::~GDOMNamedNodeMap()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

GdomNode *
gdom_named_node_map_get_named_item (GdomNamedNodeMap *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::NamedNodeMap * item = (gdom_named_node_map_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->getNamedItem(_g_name));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_named_node_map_set_named_item (GdomNamedNodeMap *thiz, GdomNode * node)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::NamedNodeMap * item = (gdom_named_node_map_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (node, 0);
    WebCore::Node * _g_node = (gdom_node_get_private(node));
    g_return_val_if_fail (_g_node, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->setNamedItem(_g_node, ec));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_named_node_map_remove_named_item (GdomNamedNodeMap *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::NamedNodeMap * item = (gdom_named_node_map_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->removeNamedItem(_g_name, ec));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_named_node_map_item (GdomNamedNodeMap *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::NamedNodeMap * item = (gdom_named_node_map_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->item(index));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_named_node_map_get_named_item_ns (GdomNamedNodeMap *thiz, gchar * namespace_uri, gchar * local_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::NamedNodeMap * item = (gdom_named_node_map_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    g_return_val_if_fail (local_name, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_local_name = WebCore::String::fromUTF8(local_name);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->getNamedItemNS(_g_namespace_uri, _g_local_name));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_named_node_map_set_named_item_ns (GdomNamedNodeMap *thiz, GdomNode * node)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::NamedNodeMap * item = (gdom_named_node_map_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (node, 0);
    WebCore::Node * _g_node = (gdom_node_get_private(node));
    g_return_val_if_fail (_g_node, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->setNamedItemNS(_g_node, ec));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_named_node_map_remove_named_item_ns (GdomNamedNodeMap *thiz, gchar * namespace_uri, gchar * local_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::NamedNodeMap * item = (gdom_named_node_map_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    g_return_val_if_fail (local_name, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_local_name = WebCore::String::fromUTF8(local_name);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->removeNamedItemNS(_g_namespace_uri, _g_local_name, ec));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "NamedNodeMap.h"
#include <gdom/GdomNamedNodeMap.h>
#include <gdom/PrivateGdomNamedNodeMap.h>
#include <gdom/PrivateIntGdomNamedNodeMap.h>
#include <gdom/GdomNamedNodeMapprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomNamedNodeMap, gdom_named_node_map, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomNamedNodeMapPrivate {
    class WebCore::NamedNodeMap *item;
};

#define GDOM_NAMED_NODE_MAP_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_NAMED_NODE_MAP, GdomNamedNodeMapPrivate))
*/
G_END_DECLS
WebCore::NamedNodeMap *
gdom_named_node_map_get_private(GdomNamedNodeMap *request)
{
    GDOMNamedNodeMap *res;
    res = static_cast<GDOMNamedNodeMap*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::NamedNodeMap*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_LENGTH,
};


static void gdom_named_node_map_finalize(GObject* object)
{
    /*GdomNamedNodeMap* request = GDOM_NAMED_NODE_MAP(object);*/
    /*GdomNamedNodeMapPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_named_node_map_parent_class)->finalize(object);
}

static void gdom_named_node_map__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomNamedNodeMap *self = GDOM_NAMED_NODE_MAP(object);
    WebCore::NamedNodeMap * gself = (gdom_named_node_map_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_named_node_map__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomNamedNodeMap *self = GDOM_NAMED_NODE_MAP(object);
    WebCore::NamedNodeMap * gself = (gdom_named_node_map_get_private(self));
    switch (prop_id) {
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_named_node_map_class_init(GdomNamedNodeMapClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_named_node_map_finalize;
    g_klass->set_property = gdom_named_node_map__set_property_fn;
    g_klass->get_property = gdom_named_node_map__get_property_fn;

    g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "named_node_map_length", /* short description */
                "read-only  gulong NamedNodeMap.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomNamedNodeMapPrivate));*/
}

static void gdom_named_node_map_init(GdomNamedNodeMap* request)
{
    /*GdomNamedNodeMapPrivate* priv = GDOM_NAMED_NODE_MAP_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

